"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.AppBuilder = void 0;
require("core-js/modules/web.dom-collections.iterator.js");
var _apolloServerCore = require("apollo-server-core");
var _apolloServerFastify = require("apollo-server-fastify");
var _fastify = _interopRequireDefault(require("fastify"));
var _tsyringe = require("tsyringe");
var _injectors = require("../injectors");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
var _class;
var _dec = (0, _tsyringe).singleton();
let AppBuilder = _class = _dec((_class = class AppBuilder {
    get config() {
        return this._appContext.configuration;
    }
    createApolloServer() {
        return new _apolloServerFastify.ApolloServer({
            schema: this._schema,
            plugins: this.config.serverPlayground ? undefined : [
                _apolloServerCore.ApolloServerPluginLandingPageDisabled
            ],
            logger: this._logger,
            introspection: this.config.serverIntrospection,
            context: async (param)=>{
                let { request  } = param;
                const ctx = {
                    ...this._appContext,
                    request: {}
                };
                for (const rcp of this._requestContextProviders){
                    const result = await rcp.apply(ctx, request);
                    ctx.request = {
                        ...ctx.request,
                        ...result
                    };
                }
                return ctx;
            },
            debug: false
        });
    }
    async start(preRun) {
        const app = (0, _fastify).default({
            logger: this._logger
        });
        await this._apolloServer.start();
        app.setErrorHandler(async (error, request, reply)=>{
            if (error.validation) {
                reply.code(400).send({
                    message: error.message,
                    validation: error.validation
                });
            } else {
                reply.code(error.statusCode || 500).send({
                    messsage: error.message
                });
            }
        });
        /* eslint-disable  @essex/adjacent-await */ await (preRun === null || preRun === void 0 ? void 0 : preRun(app));
        await app.register(this._apolloServer.createHandler({
            cors: true
        }));
        await app.listen({
            port: this.config.serverPort
        });
    }
    constructor(_appContext, _logger, _schema, _requestContextProviders){
        this._appContext = _appContext;
        this._logger = _logger;
        this._schema = _schema;
        this._requestContextProviders = _requestContextProviders;
        this._apolloServer = this.createApolloServer();
    }
}) || _class) || _class;
exports.AppBuilder = AppBuilder;
