"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.attachDirectiveResolvers = attachDirectiveResolvers;
require("core-js/modules/web.dom-collections.iterator.js");
var _utils = require("@graphql-tools/utils");
var _graphql = require("graphql");
function attachDirectiveResolvers(schema, directiveResolvers) {
    return (0, _utils).mapSchema(schema, {
        [_utils.MapperKind.OBJECT_FIELD]: (fieldConfig)=>{
            const newFieldConfig = {
                ...fieldConfig
            };
            const directives = (0, _utils).getDirectives(schema, fieldConfig);
            for (const directive of directives){
                const directiveName = directive.name;
                if (directiveResolvers[directiveName]) {
                    const resolver = directiveResolvers[directiveName];
                    const originalResolver = newFieldConfig.resolve != null ? newFieldConfig.resolve : _graphql.defaultFieldResolver;
                    const directiveArgs = directive.args;
                    newFieldConfig.resolve = (source, originalArgs, context, info)=>{
                        return resolver(()=>new Promise((resolve, reject)=>{
                                const result = originalResolver(source, originalArgs, context, info);
                                if (result instanceof Error) {
                                    reject(result);
                                }
                                resolve(result);
                            })
                        , source, directiveArgs, context, info);
                    };
                }
            }
            return newFieldConfig;
        }
    });
}
