/*!
 * Copyright (c) Microsoft. All rights reserved.
 * Licensed under the MIT license. See LICENSE file in the project.
 */ import "core-js/modules/web.dom-collections.iterator.js";
import { ApolloServerPluginLandingPageDisabled } from 'apollo-server-core';
import { ApolloServer } from 'apollo-server-fastify';
import fastify from 'fastify';
import { inject, singleton } from 'tsyringe';
import { BaseInjectorNames } from '../injectors';
var _class;
var _dec = singleton();
export let AppBuilder = _class = _dec((_class = class AppBuilder {
    get config() {
        return this._appContext.configuration;
    }
    createApolloServer() {
        return new ApolloServer({
            schema: this._schema,
            plugins: this.config.serverPlayground ? undefined : [
                ApolloServerPluginLandingPageDisabled
            ],
            logger: this._logger,
            introspection: this.config.serverIntrospection,
            context: async (param)=>{
                let { request  } = param;
                const ctx = {
                    ...this._appContext,
                    request: {}
                };
                for (const rcp of this._requestContextProviders){
                    const result = await rcp.apply(ctx, request);
                    ctx.request = {
                        ...ctx.request,
                        ...result
                    };
                }
                return ctx;
            },
            debug: false
        });
    }
    async start(preRun) {
        const app = fastify({
            logger: this._logger
        });
        await this._apolloServer.start();
        app.setErrorHandler(async (error, request, reply)=>{
            if (error.validation) {
                reply.code(400).send({
                    message: error.message,
                    validation: error.validation
                });
            } else {
                reply.code(error.statusCode || 500).send({
                    messsage: error.message
                });
            }
        });
        /* eslint-disable  @essex/adjacent-await */ await (preRun === null || preRun === void 0 ? void 0 : preRun(app));
        await app.register(this._apolloServer.createHandler({
            cors: true
        }));
        await app.listen({
            port: this.config.serverPort
        });
    }
    constructor(_appContext, _logger, _schema, _requestContextProviders){
        this._appContext = _appContext;
        this._logger = _logger;
        this._schema = _schema;
        this._requestContextProviders = _requestContextProviders;
        this._apolloServer = this.createApolloServer();
    }
}) || _class) || _class;
